--
-- ImprovedHeatplant Script
-- author: Thomas Heck (Patar)
-- date: 03.01.2017
--
-- V1.0.1.0
--
--Changelog:
-- V1.0.0.1: Problems when saving with mod-heating plants fixed.
-- V1.0.0.2: Problems in connection with rootcrop sale fixed.
-- V1.0.1.0: Freezing when joining on a dedicated server with set "Pause Game If Empty" fixed.
--			 Bug that is responsible for equal fruit-prices on dedicated server fixed.


print("--- ImprovedHeatplant by Patar loaded! (V1.0.1.0)")
ImprovedHeatplant = {};

function ImprovedHeatplant.prerequisitesPresent(specializations)
    return true
end;

function ImprovedHeatplant:load(xmlFile)
end;

function ImprovedHeatplant:writeStreamIH(streamId, connection)
	-- synchronize fillLevel on join
	if self.acceptedFillTypes[FillUtil.FILLTYPE_WOODCHIPS] and self.stationName == "station_placeableHeatingPlant" then
		if self.IH == nil then
			self:update(20);
		end;
		streamWriteFloat32(streamId, self.IH.fillLevel);
	end;
end;

function ImprovedHeatplant:readStreamIH(streamId, connection)
	-- synchronize fillLevel on join
	if self.acceptedFillTypes[FillUtil.FILLTYPE_WOODCHIPS] and self.stationName == "station_placeableHeatingPlant" then
		if self.IH == nil then
			self.IH = {};
		end;
		self.IH.fillLevel = streamReadFloat32(streamId);
	end;
end;

function ImprovedHeatplant:delete()
end;

function ImprovedHeatplant:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ImprovedHeatplant:keyEvent(unicode, sym, modifier, isDown)
end;

--[[function ImprovedHeatplant:getSaveAttributesAndNodesIH(func, nodeIdent)

	-- modified Giants-Copy for save FillLevel and priceAmount (if saved in fillHour)
	local attributes = '';
	local nodes = '';

	for fillType, _ in pairs(self.acceptedFillTypes) do
		if self.originalFillTypePrices[fillType] > 0 then
			if nodes ~= '' then
				nodes = nodes.. '\n'
			end
			local fillTypeName = FillUtil.fillTypeIntToName[fillType]

			local pricingAttributes, pricingNodes = self.pricingDynamics[fillType]:serialize(nodeIdent..'    ')
			if pricingAttributes == nil then
				pricingAttributes = ""
			end;
			if fillType == FillUtil.FILLTYPE_WOODCHIPS and self.stationName == "station_placeableHeatingPlant" then
				nodes = nodes .. nodeIdent..'<stats fillType="'..tostring(fillTypeName)..'" received="'..self.totalReceived[fillType]..'" paid="'..self.totalPaid[fillType]..'" '..pricingAttributes..' IHFillLevel="'..tostring(self.IH.fillLevel)..'" IHPriceAmount="'..tostring(self.IH.priceAmount)..'" >';               
			else
				nodes = nodes .. nodeIdent..'<stats fillType="'..tostring(fillTypeName)..'" received="'..self.totalReceived[fillType]..'" paid="'..self.totalPaid[fillType]..'" '..pricingAttributes..'>';
			end;
				
			if pricingNodes ~= nil and pricingNodes ~= "" then
				nodes = nodes.."\n"..pricingNodes;
			end;
			nodes = nodes ..'\n' .. nodeIdent .. '</stats>'
		end
	end

	return attributes, nodes;
end;


function ImprovedHeatplant:loadFromAttributesAndNodesIH(func, xmlFile, key)
	local i=0
	while true do
		local statsKey = string.format(key..".stats(%d)", i)
		if not hasXMLProperty(xmlFile, statsKey) then
			break
		end
		local fillTypeStr = getXMLString(xmlFile, statsKey.."#fillType")
		local fillType = FillUtil.fillTypeNameToInt[fillTypeStr]
		if fillType ~= nil and self.acceptedFillTypes[fillType] then
			self.totalReceived[fillType] = Utils.getNoNil(getXMLFloat(xmlFile, statsKey.."#received"), 0)
			self.totalPaid[fillType] = Utils.getNoNil(getXMLFloat(xmlFile, statsKey.."#paid"), 0)
			self.pricingDynamics[fillType]:deserialize(xmlFile, statsKey)
			-- begin custom-modification
			if fillType == FillUtil.FILLTYPE_WOODCHIPS and self.stationName == "station_placeableHeatingPlant" then
				self.IH = {};
				self.IH.fillLevel = getXMLFloat(xmlFile, statsKey.."#IHFillLevel");
				self.IH.priceAmount = getXMLFloat(xmlFile, statsKey.."#IHPriceAmount");
			end
			-- end custom-modification
		end
		i = i + 1
	end

	return true;
end; ]]


function ImprovedHeatplant:update(dt)

	-- Add improvement to Trigger
	if self.acceptedFillTypes[FillUtil.FILLTYPE_WOODCHIPS] and self.stationName == "station_placeableHeatingPlant" then
		if not self.improvementAdded then
			self.improvementAdded = true;
			if self.IH ~= nil then
				fillLevel = self.IH.fillLevel;
				priceAmount = self.IH.priceAmount;
			end;
			self.IH = {	capacity = 17560;
						fillLevel = Utils.getNoNil(fillLevel, 0);
						fillTime1 = 7;  -- 24h-system
						fillTime2 = 15; -- 24h-system
						fillTime3 = 23; -- 24h-system
						fillPerHour = 5330;  -- 1h = 3600000ms
						dtCount = 0;
						isActive = false;
						priceAmount = Utils.getNoNil(priceAmount, 0);
						playerInRange = false;
						};
						
			self.IH.allowFillType = ImprovedHeatplant.allowFillType;
			self.IH.getFreeCapacity = ImprovedHeatplant.getFreeCapacity;
			self.IH.setFillLevel = ImprovedHeatplant.setFillLevel;
			self.IH.getFillLevel = ImprovedHeatplant.getFillLevel;
			
			self:addTipTriggerTarget(self.IH, false);
		end;
		
		-- GUI
		if self.IH.playerInRange then
			local fillTypeName = FillUtil.fillTypeIndexToDesc[FillUtil.FILLTYPE_WOODCHIPS].nameI18N;
			g_currentMission:addExtraPrintText(fillTypeName .. " " ..g_i18n:getText("info_fillLevel").." "..math.floor(self.IH.fillLevel).." ("..math.floor(100*self.IH.fillLevel/self.IH.capacity).."%)");
			if self.isActive then
				g_currentMission:addExtraPrintText(g_i18n:getText("IH_isFilling"));
			end;
		end;
	end;
end;

function ImprovedHeatplant:updateTick(dt)
	if self.acceptedFillTypes[FillUtil.FILLTYPE_WOODCHIPS] and self.stationName == "station_placeableHeatingPlant" then
		if (g_currentMission.environment.currentHour == self.IH.fillTime1 or g_currentMission.environment.currentHour == self.IH.fillTime2 or g_currentMission.environment.currentHour == self.IH.fillTime3) and self.IH.fillLevel > 0 then
			if not self.isActive then
				self.isActive = true;
				self.IH.dtCount = 0;
			end;
			self.IH.dtCount = self.IH.dtCount + dt*g_currentMission.loadingScreen.missionInfo.timeScale;
			
			if self.IH.dtCount > 3600000/60 then  -- update once per minute
				self.IH.dtCount = 0;
				local deltaLevel = self.IH.fillPerHour/60;
				local deltaLevel = math.min(deltaLevel, self.IH.fillLevel)
				self.IH:setFillLevel(self.IH.fillLevel-deltaLevel);
				
				if self.isServer then
					-- modified Giants copy
					if not self.priceDropDisabled[FillUtil.FILLTYPE_WOODCHIPS] then
						self:doPriceDrop(deltaLevel, FillUtil.FILLTYPE_WOODCHIPS)
					end
					
					local desc = FillUtil.fillTypeIndexToDesc[FillUtil.FILLTYPE_WOODCHIPS]
					desc.totalAmount = desc.totalAmount + deltaLevel
					
					self.totalReceived[FillUtil.FILLTYPE_WOODCHIPS] = self.totalReceived[FillUtil.FILLTYPE_WOODCHIPS] + deltaLevel
					
					local price = deltaLevel * self:getEffectiveFillTypePrice(FillUtil.FILLTYPE_WOODCHIPS) * 1.5 -- little Bonus for waiting
					self.totalPaid[FillUtil.FILLTYPE_WOODCHIPS] = self.totalPaid[FillUtil.FILLTYPE_WOODCHIPS] + price
					self.lastIncomeName = self:getIncomeNameForFillType(FillUtil.FILLTYPE_WOODCHIPS)
					g_currentMission:addSharedMoney(price, self.lastIncomeName)
					self.IH.priceAmount = self.IH.priceAmount + price;
					if self.IH.fillLevel-deltaLevel == 0 then
						g_currentMission:addMoneyChange(self.IH.priceAmount, self.moneyChangeId);
						self.IH.priceAmount = 0; self.lastMoneyChange = 30;
					end;
				end;
			end;
		else
			if self.isActive then
				self.isActive = false;
				if self.isServer then
					g_currentMission:addSharedMoney(self.IH.priceAmount, self.lastIncomeName)
					g_currentMission:addMoneyChange(self.IH.priceAmount, self.moneyChangeId);
					self.IH.priceAmount = 0; self.lastMoneyChange = 30;
				end;
			end;
		end;
		
		-- Synchronisation of fillLevel on client-Side
		if self.IHsyncAdded == nil then
			self.lastSyncFillLevel = 0;
			self.lastSyncTime = 2000;
			self.IHsyncAdded = true;
		end;
		
		self.lastSyncTime = self.lastSyncTime - dt;
		
		if self.lastSyncTime < 0 then
			if self.lastSyncFillLevel ~= self.IH.fillLevel then
				self:raiseDirtyFlags(self.tipTriggerDirtyFlag);
				self.lastSyncFillLevel = self.IH.fillLevel;
			end;
			self.lastSyncTime = 2000;		
		end;
		
		-- GUI
		if g_currentMission.controlPlayer and g_currentMission.player ~= nil then
			local tx, ty, tz = getWorldTranslation(self.triggerId);
			local px, py, pz = getWorldTranslation(g_currentMission.player.rootNode);
			if Utils.vector3Length(px-tx, py-ty, pz-tz) < 5 then
				self.IH.playerInRange = true;
			else
				self.IH.playerInRange = false;
			end;
		elseif g_currentMission.controlledVehicle ~= nil then
			local tx, ty, tz = getWorldTranslation(self.triggerId);
			local vx, vy, vz = getWorldTranslation(g_currentMission.controlledVehicle.rootNode);
			if Utils.vector3Length(vx-tx, vy-ty, vz-tz) < 10 then
				self.IH.playerInRange = true;
			else
				self.IH.playerInRange = false;
			end;
		end;
		
	end;
end;

function ImprovedHeatplant:draw()
end;

function ImprovedHeatplant:onAttach(attacherVehicle)
end;

function ImprovedHeatplant:onDetach()
end;

-- Functions needed for tipTrigger-Target -----------------------------------------------------
function ImprovedHeatplant:allowFillType()
	return true;
end;

function ImprovedHeatplant:getFreeCapacity()
	return self.capacity - self.fillLevel;
end;

function ImprovedHeatplant:setFillLevel(fillLevel)
	self.fillLevel = fillLevel;
end;

function ImprovedHeatplant:getFillLevel()
	return self.fillLevel;
end;
-- Functions needed for tipTrigger-Target -----------------------------------------------------

function ImprovedHeatplant:getSaveAttributesAndNodesPHP(oldFunc, nodeIdent)
	-- save FillLevel and priceAmount on placableHeatPlant - something different than map-Triggers
	attributes, nodes = HeatingPlantPlaceable:superClass().getSaveAttributesAndNodes(self, nodeIdent);
	if self.tipTrigger.acceptedFillTypes[FillUtil.FILLTYPE_WOODCHIPS] and self.tipTrigger.stationName == "station_placeableHeatingPlant" then
		attributes = attributes .. ' IHFillLevel="'..self.tipTrigger.IH.fillLevel..'"';
		attributes = attributes .. ' IHPriceAmount="'..self.tipTrigger.IH.priceAmount..'"';
	end;
	return attributes, nodes;
end;

function ImprovedHeatplant:loadFromAttributesAndNodesPHP(oldFunc, xmlFile, key, resetVehicles)

	-- load FillLevel and priceAmount on placebleHeatPlant - something different than map-Triggers
    if not HeatingPlantPlaceable:superClass().loadFromAttributesAndNodes(self, xmlFile, key, resetVehicles) then
		return false;
	end;

	local IHFillLevel = getXMLFloat(xmlFile, key.."#IHFillLevel");
	if IHFillLevel ~= nil then
		self.tempIHFillLevel = IHFillLevel;
	end;
	local IHPriceAmount = getXMLFloat(xmlFile, key.."#IHPriceAmount");
	if IHPriceAmount ~= nil then
		self.tempIHPriceAmount = IHPriceAmount;
	end;

	return true;
end;

function ImprovedHeatplant:updateTickPHP(dt)

	-- Trigger is not created while loading savegame, so I had to change that values after a first Tick-Run
	if self.tipTrigger ~= nil then
		if self.tempIHFillLevel ~= nil then
			self.tipTrigger.IH.fillLevel = self.tempIHFillLevel;
			self.tipTrigger.IH.priceAmount = self.tempIHPriceAmount;
			self.tempIHFillLevel = nil;
			self.tempIHPriceAmount = nil;
		end;	
	end;
end;


	-- much functions to prepend, append and overwrite (some overwritten wasnt even existing)
	TipTrigger.update = Utils.prependedFunction(TipTrigger.update, ImprovedHeatplant.update);
	TipTrigger.updateTick = Utils.prependedFunction(TipTrigger.updateTick, ImprovedHeatplant.updateTick);
	--TipTrigger.getSaveAttributesAndNodes = Utils.overwrittenFunction(TipTrigger.getSaveAttributesAndNodes, ImprovedHeatplant.getSaveAttributesAndNodesIH);
	--TipTrigger.loadFromAttributesAndNodes = Utils.overwrittenFunction(TipTrigger.loadFromAttributesAndNodes, ImprovedHeatplant.loadFromAttributesAndNodesIH);
	TipTrigger.readStream = Utils.prependedFunction(TipTrigger.readStream, ImprovedHeatplant.readStreamIH);
	TipTrigger.writeStream = Utils.prependedFunction(TipTrigger.writeStream, ImprovedHeatplant.writeStreamIH);
	
	HeatingPlantPlaceable.getSaveAttributesAndNodes = Utils.overwrittenFunction(HeatingPlantPlaceable.getSaveAttributesAndNodes, ImprovedHeatplant.getSaveAttributesAndNodesPHP);
	HeatingPlantPlaceable.loadFromAttributesAndNodes = Utils.overwrittenFunction(HeatingPlantPlaceable.loadFromAttributesAndNodes, ImprovedHeatplant.loadFromAttributesAndNodesPHP);
	HeatingPlantPlaceable.updateTick = Utils.appendedFunction(HeatingPlantPlaceable.updateTick, ImprovedHeatplant.updateTickPHP);
